/*
 * Copyright (c) 2013, Pushkar Singh <psingh@sangoma.com>
 * Sangoma Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of the original author; nor the names of any contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * Contributors: Pushkar Singh <psingh@sangoma.com>
 */

#include "sangoma_gsm.h"

extern span_data_t SPANS[FTDM_MAX_SPANS_INTERFACE];

/**********************************************************************************
 * Fun   : ftdm_list()
 * Desc  : List all the spans configured and print their respective informations
 * Ret   : ret
 * Note  :
 * Author: Pushkar Singh
**********************************************************************************/
int ftdm_list(void)
{
	int idx = 0;
	int ret = 1;

	for (idx = 0 ; idx < FTDM_MAX_SPANS_INTERFACE; idx++) {
		ftdm_channel_t *fchan;
		ftdm_alarm_flag_t alarmbits = FTDM_ALARM_NONE;
		ftdm_signaling_status_t sigstatus;

		if (!SPANS[idx].ftdmspan) {
			continue;
		}    

		fchan = ftdm_span_get_channel(SPANS[idx].ftdmspan, 1);
		ftdm_channel_get_alarms(fchan, &alarmbits);

		if ((FTDM_SUCCESS == ftdm_span_get_sig_status(SPANS[idx].ftdmspan, &sigstatus))) {
			ret = 0;
			fprintf(stdout ,
				"+OK\n\n"
				"\n***************************************************\n"
				"span: %u (%s)\n"
				"type: %s\n"
				"physical_status: %s\n"
				"signaling_status: %s\n"
				"chan_count: %u\n"
				"inuse: %d\n"
				"***************************************************\n",
				idx,
				ftdm_span_get_name(SPANS[idx].ftdmspan),
				SPANS[idx].type,
				alarmbits ? "alarmed" : "ok",
				ftdm_signaling_status2str(sigstatus),
				ftdm_span_get_chan_count(SPANS[idx].ftdmspan),
				SPANS[idx].inuse);
		} else {
			ret = 0;
			fprintf(stdout, 
				"+OK\n\n"
				"\n***************************************************\n"
				"span: %u (%s)\n"
				"type: %s\n"
				"physical_status: %s\n"
				"chan_count: %u\n"
				"inuse: %d\n"
				"***************************************************\n",
				idx,
				ftdm_span_get_name(SPANS[idx].ftdmspan),
				SPANS[idx].type,
				alarmbits ? "alarmed" : "ok",
				ftdm_span_get_chan_count(SPANS[idx].ftdmspan),
				SPANS[idx].inuse);
		}
	}
	return ret;
}


/**********************************************************************************
 * Fun   : gsm_span_status()
 * Desc  : Prints gsm span status
 * Ret   : GSM_SUCCESS
 * Note  :
 * Author: Pushkar Singh
**********************************************************************************/
int gsm_span_status(void)
{
	char *rply = ftdm_api_execute("gsm status span_name wp3");
	if (rply) {
		fprintf(stdout, "\n\n\n***************************************");
		fprintf(stdout, "*******************************************\n");
		fprintf(stdout, "%s", rply);
		fprintf(stdout, "*********************************************");
		fprintf(stdout, "**************************************\n");
		ftdm_free(rply);
	}

	return GSM_SUCCESS;
}


/**********************************************************************************
 * Fun   : ftdm_dump()
 * Desc  : List information with respect to a span as specified by the user
 * Ret   : GSM_FAILURE | GSM_SUCCESS
 * Note  :
 * Author: Pushkar Singh
**********************************************************************************/
int ftdm_dump(void)
{
	int idx = 0;
	char choice[20];
	uint32_t span_id = 0;
	ftdm_span_t *span;
	ftdm_status_t zstatus = FTDM_FAIL;

	memset(choice, 0, sizeof(choice));

	fprintf(stdout, "\n\n\nPlease Enter the span name or span id: \n");

	if (NULL != fgets(choice, 20, stdin)) {
		if (19 < strlen(choice)) {
			fprintf(stderr, "Invalid Input...\n");
			return GSM_FAILURE;
		}
	}	

	choice[strlen(choice) - 1] = '\0';

	zstatus = ftdm_span_find_by_name(choice, &span);

	if (zstatus != FTDM_SUCCESS) {
		fprintf(stderr, "No Span with name %s is configured\n", choice);
		return GSM_FAILURE;
	}

	for (idx = 0 ; idx < FTDM_MAX_SPANS_INTERFACE; idx++) {
		ftdm_channel_t *fchan;
		ftdm_alarm_flag_t alarmbits = FTDM_ALARM_NONE;
		ftdm_signaling_status_t sigstatus;

		span_id = ftdm_span_get_id(span);

		if (!SPANS[idx].ftdmspan) {
			continue;
		}    

		fchan = ftdm_span_get_channel(SPANS[idx].ftdmspan, 1);
		ftdm_channel_get_alarms(fchan, &alarmbits);

		if (span_id != SPANS[idx].ftdmspan->span_id) {
			continue;
		}

		if ((FTDM_SUCCESS == ftdm_span_get_sig_status(SPANS[idx].ftdmspan, &sigstatus))) {
			fprintf(stdout ,
				"+OK\n\n"
				"\n***************************************************\n"
				"span: %u (%s)\n"
				"type: %s\n"
				"physical_status: %s\n"
				"signaling_status: %s\n"
				"chan_count: %u\n"
				"inuse: %d\n"
				"***************************************************\n",
				idx,
				ftdm_span_get_name(SPANS[idx].ftdmspan),
				SPANS[idx].type,
				alarmbits ? "alarmed" : "ok",
				ftdm_signaling_status2str(sigstatus),
				ftdm_span_get_chan_count(SPANS[idx].ftdmspan),
				SPANS[idx].inuse);
			break;
		} else {
			fprintf(stdout, 
				"+OK\n"
				"\n***************************************************\n"
				"span: %u (%s)\n"
				"type: %s\n"
				"physical_status: %s\n"
				"chan_count: %u\n"
				"inuse: %d\n"
				"***************************************************\n",
				idx,
				ftdm_span_get_name(SPANS[idx].ftdmspan),
				SPANS[idx].type,
				alarmbits ? "alarmed" : "ok",
				ftdm_span_get_chan_count(SPANS[idx].ftdmspan),
				SPANS[idx].inuse);
			break;
		}
	}

	return GSM_SUCCESS;
}
